﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.CodeDom.Compiler;

namespace Compiler.Compiler.VisualBasic
{
    public class FragmentCompiler:ICompiler
    {
        public bool Compile(string taskName, string code, string language, List<ParamInfo> paramInfos, List<string> references, List<string> usings,out List<string> message)
        {
            if (string.IsNullOrWhiteSpace(code))
                throw new ArgumentNullException();
            if (string.IsNullOrWhiteSpace(language))
                throw new ArgumentNullException();

            message = new List<string>();

            StringBuilder source = new StringBuilder();
            List<string> defaultUsings = DefaultValueGetter.GetDefaultUsingNamespaces();
            defaultUsings.ForEach(d => source.AppendLine(string.Format("Imports {0}", d)));
            if (usings != null && usings.Count > 0)
                usings.ForEach(u =>
                {
                    if (!defaultUsings.Contains(u))
                        source.AppendLine(string.Format("Imports {0}", u));
                });

            source.AppendLine(string.Format("Public Class {0}", taskName));
            source.AppendLine("Inherits Task");
            if (paramInfos != null && paramInfos.Count > 0)
            {
                paramInfos.ForEach(p => source.AppendLine(string.Format("Public Property {0} As {1}", p.Name, p.Type)));
            }
            source.AppendLine("Public Overrides Function Execute() As Boolean");
            source.AppendLine(code);
            source.AppendLine("Return True");
            source.AppendLine("End Function");
            source.AppendLine("End Class");

            CompilerParameters parameters = new CompilerParameters();
            parameters.GenerateExecutable = false;
            List<string> defaultReferencedAssemblies = DefaultValueGetter.GetDefaultReferencedAssemblies();
            defaultReferencedAssemblies.ForEach(d => parameters.ReferencedAssemblies.Add(d));
            if (references != null && references.Count > 0)
                references.ForEach(r =>
                {
                    if (!defaultReferencedAssemblies.Contains(r))
                        parameters.ReferencedAssemblies.Add(r);
                });
            
            CodeDomProvider provider = CodeDomProvider.CreateProvider(language);
            CompilerResults result= provider.CompileAssemblyFromSource(parameters, source.ToString());
            if (result.Errors.Count > 0)
            {
                foreach (CompilerError error in result.Errors)
                    message.Add(error.ErrorText);
                return false;
            }
            else
                return true;
            
        }
    }
}
