﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Intellisense;
using System.IO;

namespace IntellisenseTest
{
    [TestClass]
    public class DisplayManagerTest
    {
        #region IsDisplay
        [TestMethod]
        public void IsDisplayTestMethod1()
        {
            bool result = DisplayManager.IsDisplay("", 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsDisplayTestMethod2()
        {
            bool result = DisplayManager.IsDisplay(null, 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsDisplayTestMethod3()
        {
            bool result = DisplayManager.IsDisplay("aaa", 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsDisplayTestMethod4()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 0);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// xml declaration
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod5()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 37);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// xml declaration
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod6()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 40);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// Element
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod7()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 41);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// Element
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod8()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 42);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// Element
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod9()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 48);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// Element
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod10()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 49);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// Element Attribute
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod11()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 50);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// Element Attribute
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod12()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 67);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// Element Attribute
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod13()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 69);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// CDATA
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod14()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 370);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// CDATA
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod15()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 371);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// CDATA
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod16()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 388);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// CDATA
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod17()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 389);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// comment
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod18()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 2542);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// comment
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod19()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 2543);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// comment
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod20()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 2810);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// comment
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod21()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 2811);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// endelement
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod22()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 389);
            Assert.IsTrue(result);
        }

        /// <summary>
        /// endelement
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod23()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 390);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// endelement
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod24()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 395);
            Assert.IsFalse(result);
        }

        /// <summary>
        /// endelement
        /// </summary>
        [TestMethod]
        public void IsDisplayTestMethod25()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }
            bool result = DisplayManager.IsDisplay(target, 396);
            Assert.IsTrue(result);
        }
        #endregion

        [TestMethod]
        public void IsTaskTestMethod1()
        {
            bool result = DisplayManager.IsTask("", 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsTaskTestMethod2()
        {
            bool result = DisplayManager.IsTask(null, 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsTaskTestMethod3()
        {
            bool result = DisplayManager.IsTask("aaa", 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsTaskTestMethod4()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsTask(target, 0);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsTaskTestMethod5()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsTask(target, 40);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsTaskTestMethod6()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsTask(target, 41);
            Assert.IsTrue(result);
        }

        [TestMethod]
        public void IsTaskTestMethod7()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsTask(target, 48);
            Assert.IsTrue(result);
        }

        [TestMethod]
        public void IsTaskTestMethod8()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsTask(target, 49);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsPropertyTestMethod1()
        {

            bool result = DisplayManager.IsProperty(null, 49);
            Assert.IsFalse(result);
        }
        [TestMethod]
        public void IsPropertyTestMethod2()
        {

            bool result = DisplayManager.IsProperty("", 49);
            Assert.IsFalse(result);
        }
        [TestMethod]
        public void IsPropertyTestMethod3()
        {

            bool result = DisplayManager.IsProperty("aa", 49);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsPropertyTestMethod4()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsProperty(target, 49);
            Assert.IsTrue(result);
        }

        [TestMethod]
        public void IsPropertyTestMethod5()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsProperty(target, 48);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsPropertyTestMethod6()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsProperty(target, 151);
            Assert.IsFalse(result);
        }

        [TestMethod]
        public void IsPropertyTestMethod7()
        {
            string target = null;
            using (StreamReader reader = new StreamReader("TestData\\XMLFile1.xml"))
            {
                target = reader.ReadToEnd();
            }

            bool result = DisplayManager.IsProperty(target, 150);
            Assert.IsTrue(result);
        }

    }
}
